# %%
import anltk


print(anltk.__version__)

print(anltk.__doc__)

print(anltk.replace('بسم الله',
 {
	  'ل' : 'ر' ,
 	  ' ' : 'd'
 }
))

print(anltk.replace_str('بسم الله',
 {
	 'ال' : 'الْ', 
	 'بس' : 'ييي', 

 }
))

# %%





from anltk_pybind.constants import HAMZA, ALEF_MADDA, ALEF_HAMZA_ABOVE, WAW_HAMZA_ABOVE, ALEF_HAMZA_BELOW, YEH_HAMZA_ABOVE, ALEF_NO_HAMZA, BEH, TEH_MARBOOTA, TEH, THEH, JEEM, HAH, KHAH, DAL, THAL, REH, ZAAY, SEEN, SHEEN, SAD, DAD, TAH, ZAH, AIN, GHAIN, FEH, QAF, KAF, LAM, MEEM, NOON, HEH, WAW, ALEF_MAQSURA, YEH, ALEF_WASLA, PEH, TCHEH, VEH, GAF, TATWEEL, TANWEEN_FATHA, TANWEEN_DAMMA, TANWEEN_KASRA, FATHA, DAMMA, KASRA, SHADDA, SUKUN, FASLA, QUESTION_MARK, FASLA_MANKOOTA, NOKTATAN, NESBA, TA3AJJOB, KAWS_MAFTOOH, KAWS_MAKFOOL, DARB, JAM3, KESMA, TAR7, YOSAWI, NOQTA, SEFR, WAHED, ETHNAN, THALATHA, ARBA3A, KHAMSA, SETTA, SAB3A, THAMANEYA, TES3A, SMALL_ALEF, SMALL_WAW, SMALL_YEH

from anltk_pybind.constants import HARAKAT, ARQAM, ALPHABET, QAMARI, SHAMSI





from . import constants
from .version import __version__
from .anltk import \
    AR2BW, BW2AR, AR2SBW, SBW2AR, \
    tafqeet, \
    transliterate, \
    tokenize_words, \
    is_tashkeel, \
    is_arabic_alpha, \
    is_small, \
    is_indic_digit, \
    is_shamsi, \
    is_qamari, \
    remove_tashkeel, \
    remove_small, \
    remove_non_alpha, \
    remove_non_alphanumeric, \
    remove_non_alphanumeric_and_tashkeel, \
    remove_kasheeda, \
    normalize_hamzat, \
    duplicate_shadda_letter, \
    remove_if, \
    replace, \
    replace_str , \
    split, \
    fold_white_spaces ,\
    fold_if




from typing import Callable, Dict, List
import anltk_pybind


from anltk_pybind import AR2BW, BW2AR, AR2SBW, SBW2AR


def tafqeet(num: int, is_ordinal: bool = False,  is_feminine: bool = False) -> str:
    """Converts a number into Arabic spoken form

    Args:
        num (int): input number
        is_ordinal (bool, optional): الأول if True, واحد if false. Defaults to False.
        is_feminine (bool, optional): الخامسة if True, الخامس if False. Defaults to False.

    Returns:
        [str]
    Example: 
        `anltk.tafqeet(1934)  ` \n
        `'ألف و تسعمئة و أربعة و ثلاثون'` 

    """
    return anltk_pybind.tafqeet(num, is_ordinal, is_feminine)


def transliterate(text: str, mapping: anltk_pybind.CharMapping) -> str:
    """ Convert between famous characters mapping

    Args:
        text (str): input text
        mapping (anltk_pybind.CharMapping): Mapping type 
            AR2BW : Arabic to buckwalter
            AR2SBW : Arabic to safe buckwalter
            BW2AR: Buckwalter to arabic
            SBW2AR : Safe buckwalter to arabic
    Returns:
        [str]
    Example : 
        `anltk.transliterate("أبجد هوز حطي كلمن سعفص قرشت ثخذ ضظغ", anltk.AR2BW) `\n 
        `'>bjd hwz HTy klmn sEfS qr$t vx* DZg ` 
    """
    return anltk_pybind.transliterate(text, mapping)


def tokenize_words(text: str) -> List[str]:
    return anltk_pybind.tokenize_words(text)


def is_tashkeel(c: str) -> bool:
    return anltk_pybind.is_tashkeel(c)


def is_arabic_alpha(c: str) -> bool:
    return anltk_pybind.is_arabic_alpha(c)


def is_small(c: str) -> bool:
    return anltk_pybind.is_small(c)


def is_indic_digit(c: str) -> bool:
    return anltk_pybind.is_indic_digit(c)


def is_valid_kalima(c: str) -> bool:
    return anltk_pybind.is_valid_kalima(c)


def is_shamsi(c: str) -> bool:
    return anltk_pybind.is_shamsi(c)


def is_qamari(c: str) -> bool:
    return anltk_pybind.is_qamari(c)


def remove_tashkeel(text: str) -> str:
    return anltk_pybind.remove_tashkeel(text)


def remove_small(text: str) -> str:
    return anltk_pybind.remove_small(text)


def remove_non_alpha(text: str, stop_list: str = ' ') -> str:
    return anltk_pybind.remove_non_alpha(text, stop_list)


def remove_non_alphanumeric(text: str, stop_list: str = ' ') -> str:
    return anltk_pybind.remove_non_alphanumeric(text, stop_list)


def remove_non_alphanumeric_and_tashkeel(
        text: str, stop_list: str = ' ') -> str:
    return anltk_pybind.remove_non_alphanumeric_and_tashkeel(text, stop_list)


def remove_kasheeda(text: str) -> str:
    return anltk_pybind.remove_kasheeda(text)


def normalize_hamzat(text: str) -> str:
    return anltk_pybind.normalize_hamzat(text)


def duplicate_shadda_letter(text: str) -> str:
    return anltk_pybind.duplicate_shadda_letter(text)


def remove_if(text: str, stop_list: str, func : Callable) -> str:
    return anltk_pybind.remove_if(text, stop_list, func)


def replace(text : str, chars_map : Dict[str, str]):
    return anltk_pybind.replace(text, chars_map)

def replace_str(text : str, chars_map : Dict[str, str]):
    return anltk_pybind.replace_str(text, chars_map)




def fold_if(text : str, func : Callable) -> str:
    return anltk_pybind.fold_if(text, func)

def fold_white_spaces(text : str) -> str:
    return anltk_pybind.fold_white_spaces(text)


def split(text : str, delimeters :str = ' ', keep_delimeters : bool = False) -> List[str]:
    return anltk_pybind.split(text, delimeters, keep_delimeters)


import os
import sys
import setuptools
import shutil
import glob
import platform
import pybind11
from pybind11.setup_helpers import Pybind11Extension, build_ext

# Figure out environment for cross-compile
anltk_source = os.getenv(
    "ANLTK_SOURCE", os.path.abspath(os.path.dirname(__file__)))
system = os.environ.get('ANLTK_PLATFORM', platform.system())
architecture = os.environ.get('ANLTK_ARCHITECTURE', platform.architecture()[0])

# Create OS-dependent, but Python-independent wheels.
try:
    from wheel.bdist_wheel import bdist_wheel
except ImportError:
    cmdclass = {}
else:
    class bdist_wheel_tag_name(bdist_wheel):
        def get_tag(self):
            abi = 'none'
            if system == 'Darwin':
                oses = 'macosx_10_6_x86_64'
            elif system == 'Windows' and architecture == '32bit':
                oses = 'win32'
            elif system == 'Windows' and architecture == '64bit':
                oses = 'win_amd64'
            elif system == 'Linux' and architecture == '64bit':
                oses = 'linux_x86_64'
            elif system == 'Linux':
                oses = 'linux_' + architecture
            else:
                raise TypeError("Unknown build environment")
            return 'py3', abi, oses
    cmdclass = {'bdist_wheel': bdist_wheel_tag_name}

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()


def _get_project_version():
    version_path = os.path.join(
        anltk_source, "python", "anltk_module", "anltk", "version.py")
    version = {}
    with open(version_path, encoding="utf-8") as fp:
        exec(fp.read(), version)
    return version["__version__"]


include_dirs = [pybind11.get_include(),
                os.path.join(anltk_source, 'anltk', 'include'),
                os.path.join(anltk_source, 'anltk', 'third_party', 'utfcpp', 'source')]


cflags = ['-O3']
ldflags = []
if system == 'Windows':
    cflags = ['/O2', '/utf-8']

anltk_src_files = map(str, os.listdir(
    os.path.join(anltk_source, 'anltk', 'src')))
anltk_src_cpp = list(filter(lambda x: x.endswith('.cpp'), anltk_src_files))
anltk_src_cpp = list(
    map(lambda x: str(os.path.join(anltk_source, 'anltk', 'src', x)), anltk_src_cpp)
)

ext_modules = [
    Pybind11Extension("anltk_pybind",
                      sources=[os.path.join(
                          anltk_source, 'python', 'anltk_module', 'anltk',
                          'pybind', 'anltk_pybind.cpp')] + anltk_src_cpp,
                      extra_compile_args=cflags,
                      cxx_std=17,
                      extra_link_args=ldflags,
                      include_dirs=include_dirs,
                      define_macros=[(("UTF_CPP_CPLUSPLUS", "201703L"))]
                      )
]

setuptools.setup(
    name="anltk",
    version=_get_project_version(),
    author="Abdullah Alattar",
    author_email="abdullah.mohammad.alattar@gmail.com",
    description="Arabic language processing toolkit",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Abdullah-AlAttar/anltk",
    cmdclass=cmdclass,
    python_requires='>=3',
    project_urls={
        "Source": "https://github.com/Abdullah-AlAttar/anltk",
    },
    ext_modules=ext_modules,
    zip_safe=False,
    include_package_data=False,
    package_dir={str(''): str('python/anltk_module')},
    packages=[
        str('anltk'),
        str('anltk.constants'),
        str('anltk.tests')
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Natural Language :: Arabic',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: MacOS',
    ],
    keywords = ['NLP', 'Arabic', 'python', 'arabic', 'c++']
)



import anltk


# def test_has_spaces():

#     ar_text = "أبجد هوز حطي كلمن سعفص قرشت ثخذ ضظغ"
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_empty():

#     ar_text = ""
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_Has_double_consecutive_shadda():

#     ar_text = "بسّّم"
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_first_character_tashkeel():

#     ar_text = "ًبسم"
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_First_Character_WAW_HAMZA():

#     ar_text = "ؤامر"
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_First_Character_YEH_HAMZA():

#     ar_text = "ئاسر"
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_Alef_Maqsura_not_last_Character():

#     ar_text = "ىا ليلى"
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_Teh_Marbuta_not_last_Character():

#     ar_text = "ىا لةيلى"
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_Three_consectuve_harakat():

#     ar_text = "ىاًَُ لةيلى"
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_double_Teh_Marbuta():

#     ar_text = "ىا لةةيلى"
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_Valid():

#     ar_text = "فأسقيناكموه"
#     assert anltk.is_valid_kalima(ar_text) == True

# def test_Valid1():

#     ar_text = "فأسًقيٍنََاّكمٍوُه"
#     assert anltk.is_valid_kalima(ar_text) == True


def test_tokenize():

    text = "بسم الله@ الرحمن23 الرحيم."

    assert anltk.tokenize_words(
        text) == ["بسم", "الله", "@", "الرحمن", "23", "الرحيم", "."]


def test_tafqeet():

    assert anltk.tafqeet(394) == "ثلاثمئة و أربعة و تسعون"


def test_transliterate():

    assert anltk.transliterate("أبجد هوز حطي كلمن سعفص قرشت ثخذ ضظغ",
                               anltk.AR2BW) == '>bjd hwz HTy klmn sEfS qr$t vx* DZg'

    # Safe Buckwalter
    assert anltk.transliterate("أبجد هوز حطي كلمن سعفص قرشت ثخذ ضظغ",
                               anltk.AR2SBW) == 'Objd hwz HTy klmn sEfS qrct vxV DZg'

    assert anltk.transliterate(">bjd hwz HTy klmn sEfS qr$t vx* DZg",
                               anltk.BW2AR) == 'أبجد هوز حطي كلمن سعفص قرشت ثخذ ضظغ'

    assert anltk.transliterate("Objd hwz HTy klmn sEfS qrct vxV DZg",
                               anltk.SBW2AR) == 'أبجد هوز حطي كلمن سعفص قرشت ثخذ ضظغ'

def test_remove_tashkeel():
    assert anltk.remove_tashkeel("فرَاشَةٌ مُلَوَّنَةٌ") == "فراشة ملونة"


def test_remove_non_alpha():
    assert anltk.remove_non_alpha('فرَاdشَةٌ مُلَوَّنَ3ةٌ!', stop_list = '! ') ==  'فراشة ملونة!'
    assert anltk.remove_non_alphanumeric('فرَاdشَةٌ مُلَوَّ3نَةٌ!', stop_list = '! ') ==  'فراشة ملو3نة!'

    assert anltk.remove_non_alphanumeric_and_tashkeel('فرَاdشَةٌ مُلَوَّ3نَةٌ!', stop_list = '! ') == 'فرَاشَةٌ مُلَوَّ3نَةٌ!'
    assert anltk.remove_kasheeda('بســــم الـــله') ==  'بسم الله'


def test_remove_if():
    assert anltk.remove_if('باب الحارة', " ", lambda x : x == 'ب') == 'ا الحارة'

def test_folding():
    assert anltk.fold_white_spaces('بسم     الله    الرحمن ال   رحيم\t\t') == 'بسم الله الرحمن ال رحيم\t'

def test_splitting():
    assert anltk.split("ffsl بسم اللهْ") == ['ffsl', 'بسم', 'اللهْ']
    assert anltk.split("يا الله, يا رحمن يا رحيم." , delimeters=".," , keep_delimeters=False) == ['يا الله', ' يا رحمن يا رحيم']
    assert anltk.split("يا الله, يا رحمن يا رحيم." , delimeters=".," , keep_delimeters=True) == ['يا الله,', ' يا رحمن يا رحيم.']